@@ -56,7 +56,7 @@ const selectors = {
     count: '[data-collapse="count"]',
     placeholder: '.collapsecolumndropdown [data-region="placeholder"]',
     fullDropdown: '.collapsecolumndropdown',
-    cellMenuButton: '.cellmenubtn',
+    searchResultContainer: '.searchresultitemscontainer',
 };
 
 const countIndicator = document.querySelector(selectors.count);
@@ -99,6 +99,19 @@ export default class ColumnSearch extends search_combobox {
                 document.querySelector('.gradereport-grader-table').classList.remove('d-none');
             }, 10);
         }).then(() => pendingPromise.resolve()).catch(Notification.exception);
+
+        this.$component.on('hide.bs.dropdown', () => {
+            const searchResultContainer = this.component.querySelector(selectors.searchResultContainer);
+            searchResultContainer.scrollTop = 0;
+
+            // Use setTimeout to make sure the following code is executed after the click event is handled.
+            setTimeout(() => {
+                if (this.searchInput.value !== '') {
+                    this.searchInput.value = '';
+                    this.searchInput.dispatchEvent(new Event('input', {bubbles: true}));
+                }
+            });
+        });
     }
 
     /**
@@ -119,15 +132,6 @@ export default class ColumnSearch extends search_combobox {
         return '.searchresultitemscontainer';
     }
 
-    /**
-     * The triggering div that contains the searching widget.
-     *
-     * @returns {string}
-     */
-    triggerSelector() {
-        return '.collapsecolumn';
-    }
-
     /**
      * Return the dataset that we will be searching upon.
      *
@@ -192,7 +196,6 @@ export default class ColumnSearch extends search_combobox {
     async docClickHandler(e) {
         if (e.target.dataset.hider === selectors.hider) {
             e.preventDefault();
-            const pendingPromise = new Pending('gradereport_grader/collapse:docClickHandler:hide');
             const desiredToHide = e.target.closest(selectors.colVal) ?
                 e.target.closest(selectors.colVal)?.dataset.col :
                 e.target.closest(selectors.itemVal)?.dataset.itemid;
@@ -202,13 +205,11 @@ export default class ColumnSearch extends search_combobox {
             }
             await this.prefcountpipe();
 
-            await this.nodesUpdate(desiredToHide);
-            pendingPromise.resolve();
+            this.nodesUpdate(desiredToHide);
         }
 
         if (e.target.closest('button')?.dataset.hider === selectors.expand) {
             e.preventDefault();
-            const pendingPromise = new Pending('gradereport_grader/collapse:docClickHandler:expand');
             const desiredToHide = e.target.closest(selectors.colVal) ?
                 e.target.closest(selectors.colVal)?.dataset.col :
                 e.target.closest(selectors.itemVal)?.dataset.itemid;
@@ -217,28 +218,8 @@ export default class ColumnSearch extends search_combobox {
 
             await this.prefcountpipe();
 
-            await this.nodesUpdate(e.target.closest(selectors.colVal)?.dataset.col);
-            await this.nodesUpdate(e.target.closest(selectors.colVal)?.dataset.itemid);
-            pendingPromise.resolve();
-        }
-    }
-
-    /**
-     * The handler for when a user presses a key within the component.
-     *
-     * @param {KeyboardEvent} e The triggering event that we are working with.
-     */
-    async keyHandler(e) {
-        super.keyHandler(e);
-
-        // Switch the key presses to handle keyboard nav.
-        switch (e.key) {
-            case 'Tab':
-                if (e.target.closest(this.selectors.input)) {
-                    e.preventDefault();
-                    this.clearSearchButton.focus({preventScroll: true});
-                }
-                break;
+            this.nodesUpdate(e.target.closest(selectors.colVal)?.dataset.col);
+            this.nodesUpdate(e.target.closest(selectors.colVal)?.dataset.itemid);
         }
     }
 
@@ -350,14 +331,11 @@ export default class ColumnSearch extends search_combobox {
         });
     }
 
-    async nodesUpdate(item) {
+    nodesUpdate(item) {
         const colNodesToHide = [...document.querySelectorAll(`[data-col="${item}"]`)];
         const itemIDNodesToHide = [...document.querySelectorAll(`[data-itemid="${item}"]`)];
-        const elements = [...colNodesToHide, ...itemIDNodesToHide];
-        if (elements && elements.length) {
-            const pendingPromise = new Pending('gradereport_grader/collapse:nodesUpdate:' + item);
-            this.updateDisplay(elements).then(() => pendingPromise.resolve()).catch(Notification.exception);
-        }
+        this.nodes = [...colNodesToHide, ...itemIDNodesToHide];
+        this.updateDisplay();
     }
 
     /**
@@ -422,68 +400,58 @@ export default class ColumnSearch extends search_combobox {
 
     /**
      * With an array of nodes, switch their classes and values.
-     *
-     * @param {Array} elements The elements to update.
      */
-    async updateDisplay(elements) {
-        const promises = [];
-        elements.forEach((element) => {
-            promises.push(this.updateDisplayForElement(element));
-        });
+    updateDisplay() {
+        this.nodes.forEach((element) => {
+            const content = element.querySelector(selectors.content);
+            const sort = element.querySelector(selectors.sort);
+            const expandButton = element.querySelector(selectors.expandbutton);
+            const rangeRowCell = element.querySelector(selectors.rangerowcell);
+            const avgRowCell = element.querySelector(selectors.avgrowcell);
+            const nodeSet = [
+                element.querySelector(selectors.menu),
+                element.querySelector(selectors.icons),
+                content
+            ];
 
-        await Promise.all(promises);
-    }
+            // This can be further improved to reduce redundant similar calls.
+            if (element.classList.contains('cell')) {
+                // The column is actively being sorted, lets reset that and reload the page.
+                if (sort !== null) {
+                    window.location = this.defaultSort;
+                }
+                if (content === null) {
+                    // If it's not a content cell, it must be an overall average or a range cell.
+                    const rowCell = avgRowCell ?? rangeRowCell;
 
-    /**
-     * Update display for given element, switch its classes and values.
-     *
-     * @param {HTMLElement} element The element to update.
-     */
-    async updateDisplayForElement(element) {
-        const content = element.querySelector(selectors.content);
-        const sort = element.querySelector(selectors.sort);
-        const expandButton = element.querySelector(selectors.expandbutton);
-        const rangeRowCell = element.querySelector(selectors.rangerowcell);
-        const avgRowCell = element.querySelector(selectors.avgrowcell);
-        const cellMenuButton = element.querySelector(selectors.cellMenuButton);
-        const nodeSet = [
-            element.querySelector(selectors.menu),
-            element.querySelector(selectors.icons),
-            content
-        ];
-
-        // This can be further improved to reduce redundant similar calls.
-        if (element.classList.contains('cell')) {
-            // The column is actively being sorted, lets reset that and reload the page.
-            if (sort !== null) {
-                window.location = this.defaultSort;
-            }
-            if (content === null) {
-                // If it's not a content cell, it must be an overall average or a range cell.
-                const rowCell = avgRowCell ?? rangeRowCell;
-
-                rowCell?.classList.toggle('d-none');
-            } else if (content.classList.contains('d-none')) {
-                // We should always have content but some cells do not contain menus or other actions.
-                element.classList.remove('collapsed');
-                // If there are many nodes, apply the following.
-                if (content.childNodes.length > 1) {
-                    content.classList.add('d-flex');
+                    rowCell?.classList.toggle('d-none');
+                    rowCell?.setAttribute('aria-hidden',
+                        rowCell?.classList.contains('d-none') ? 'true' : 'false');
+                } else if (content.classList.contains('d-none')) {
+                    // We should always have content but some cells do not contain menus or other actions.
+                    element.classList.remove('collapsed');
+                    // If there are many nodes, apply the following.
+                    if (content.childNodes.length > 1) {
+                        content.classList.add('d-flex');
+                    }
+                    nodeSet.forEach(node => {
+                        node?.classList.remove('d-none');
+                        node?.setAttribute('aria-hidden', 'false');
+                    });
+                    expandButton?.classList.add('d-none');
+                    expandButton?.setAttribute('aria-hidden', 'true');
+                } else {
+                    element.classList.add('collapsed');
+                    content.classList.remove('d-flex');
+                    nodeSet.forEach(node => {
+                        node?.classList.add('d-none');
+                        node?.setAttribute('aria-hidden', 'true');
+                    });
+                    expandButton?.classList.remove('d-none');
+                    expandButton?.setAttribute('aria-hidden', 'false');
                 }
-                nodeSet.forEach(node => {
-                    node?.classList.remove('d-none');
-                });
-                expandButton?.classList.add('d-none');
-                cellMenuButton?.focus();
-            } else {
-                element.classList.add('collapsed');
-                content.classList.remove('d-flex');
-                nodeSet.forEach(node => {
-                    node?.classList.add('d-none');
-                });
-                expandButton?.classList.remove('d-none');
             }
-        }
+        });
     }
 
     /**
@@ -510,6 +478,7 @@ export default class ColumnSearch extends search_combobox {
         // Update the collapsed button pill.
         this.countUpdate();
         const {html, js} = await renderForPromise('gradereport_grader/collapse/collapsebody', {
+            'instance': this.instance,
             'results': this.getMatchedResults(),
             'userid': this.userID,
         });
@@ -523,6 +492,7 @@ export default class ColumnSearch extends search_combobox {
         // Add a small BS listener so that we can set the focus correctly on open.
         this.$component.on('shown.bs.dropdown', () => {
             this.searchInput.focus({preventScroll: true});
+            this.selectallEnable();
         });
     }
 
@@ -530,14 +500,26 @@ export default class ColumnSearch extends search_combobox {
      * Build the content then replace the node.
      */
     async renderDropdown() {
-        const form = this.component.querySelector(selectors.formDropdown);
-        const selectall = form.querySelector('[data-action="selectall"]');
         const {html, js} = await renderForPromise('gradereport_grader/collapse/collapseresults', {
+            instance: this.instance,
             'results': this.getMatchedResults(),
             'searchTerm': this.getSearchTerm(),
         });
-        selectall.disabled = this.getMatchedResults().length === 0;
         replaceNodeContents(this.getHTMLElements().searchDropdown, html, js);
+        this.selectallEnable();
+        // Reset the expand button to be disabled as we have re-rendered the dropdown.
+        const form = this.component.querySelector(selectors.formDropdown);
+        const expandButton = form.querySelector(`[data-action="${selectors.formItems.save}"`);
+        expandButton.disabled = true;
+    }
+
+    /**
+     * Given we render the dropdown, Determine if we want to enable the select all checkbox.
+     */
+    selectallEnable() {
+        const form = this.component.querySelector(selectors.formDropdown);
+        const selectall = form.querySelector('[data-action="selectall"]');
+        selectall.disabled = this.getMatchedResults().length === 0;
     }
 
     /**
