@@ -158,14 +158,14 @@ var CSS = {
                 // Add the size entry boxes.
                 '<div class="mb-1">' +
                 '<label class="" for="{{elementid}}_{{CSS.INPUTSIZE}}">{{get_string "size" component}}</label>' +
-                '<div id="{{elementid}}_{{CSS.INPUTSIZE}}" class="form-inline {{CSS.INPUTSIZE}}">' +
+                '<div id="{{elementid}}_{{CSS.INPUTSIZE}}" class="d-flex flex-wrap align-items-center {{CSS.INPUTSIZE}}">' +
                 '<label class="accesshide" for="{{elementid}}_{{CSS.INPUTWIDTH}}">{{get_string "width" component}}</label>' +
-                '<input type="text" class="form-control mr-1 input-mini {{CSS.INPUTWIDTH}}" ' +
+                '<input type="text" class="form-control w-auto mr-1 input-mini {{CSS.INPUTWIDTH}}" ' +
                 'id="{{elementid}}_{{CSS.INPUTWIDTH}}" size="4"/> x' +
 
                 // Add the height entry box.
                 '<label class="accesshide" for="{{elementid}}_{{CSS.INPUTHEIGHT}}">{{get_string "height" component}}</label>' +
-                '<input type="text" class="form-control ml-1 input-mini {{CSS.INPUTHEIGHT}}" ' +
+                '<input type="text" class="form-control w-auto ml-1 input-mini {{CSS.INPUTHEIGHT}}" ' +
                 'id="{{elementid}}_{{CSS.INPUTHEIGHT}}" size="4"/>' +
 
                 // Add the constrain checkbox.
@@ -179,9 +179,9 @@ var CSS = {
                 '</div>' +
 
                 // Add the alignment selector.
-                '<div class="form-inline mb-1">' +
-                '<label class="for="{{elementid}}_{{CSS.INPUTALIGNMENT}}">{{get_string "alignment" component}}</label>' +
-                '<select class="custom-select {{CSS.INPUTALIGNMENT}}" id="{{elementid}}_{{CSS.INPUTALIGNMENT}}">' +
+                '<div class="d-flex flex-wrap align-items-center mb-1">' +
+                '<label class="mb-0" for="{{elementid}}_{{CSS.INPUTALIGNMENT}}">{{get_string "alignment" component}}</label>' +
+                '<select class="custom-select ml-2 {{CSS.INPUTALIGNMENT}}" id="{{elementid}}_{{CSS.INPUTALIGNMENT}}">' +
                     '{{#each alignments}}' +
                         '<option value="{{value}}">{{get_string str ../component}}</option>' +
                     '{{/each}}' +
@@ -365,7 +365,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
 
         // Trigger form upload start events.
         require(['core_form/events'], function(FormEvent) {
-            FormEvent.triggerUploadStarted(self.editor.get('id'));
+            FormEvent.notifyUploadStarted(self.editor.get('id'));
         });
 
         var options = host.get('filepickeroptions').image,
@@ -424,7 +424,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                             }
                             // Trigger form upload complete events.
                             require(['core_form/events'], function(FormEvent) {
-                                FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                                FormEvent.notifyUploadCompleted(self.editor.get('id'));
                             });
                             throw new M.core.ajaxException(result);
                         }
@@ -455,7 +455,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                     Y.use('moodle-core-notification-alert', function() {
                         // Trigger form upload complete events.
                         require(['core_form/events'], function(FormEvent) {
-                            FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                            FormEvent.notifyUploadCompleted(self.editor.get('id'));
                         });
                         new M.core.alert({message: M.util.get_string('servererror', 'moodle')});
                     });
@@ -465,7 +465,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                 }
                 // Trigger form upload complete events.
                 require(['core_form/events'], function(FormEvent) {
-                    FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                    FormEvent.notifyUploadCompleted(self.editor.get('id'));
                 });
             }
         };
