@@ -368,7 +368,7 @@ class api {
             $settings->enabledashboard = $CFG->enabledashboard;
         }
 
-        if (empty($section) || $section === 'themesettings') {
+        if (empty($section) || ($section === 'themesettings' || $section === 'themesettingsadvanced')) {
             $settings->customusermenuitems = $CFG->customusermenuitems;
         }
 
@@ -388,6 +388,25 @@ class api {
                 $CFG->searchbanner, FORMAT_HTML, $context)[0];
         }
 
+        if (empty($section) || $section === 'privacysettings') {
+            $settings->tool_dataprivacy_contactdataprotectionofficer = get_config('tool_dataprivacy', 'contactdataprotectionofficer');
+            $settings->tool_dataprivacy_showdataretentionsummary = get_config('tool_dataprivacy', 'showdataretentionsummary');
+        }
+
+        if (empty($section) || $section === 'blog') {
+            $settings->useblogassociations = $CFG->useblogassociations;
+            $settings->bloglevel = $CFG->bloglevel;
+            $settings->blogusecomments = $CFG->blogusecomments;
+        }
+
+        if (empty($section) || $section === 'h5psettings') {
+            \core_h5p\local\library\autoloader::register();
+            $customcss = \core_h5p\file_storage::get_custom_styles();
+            if (!empty($customcss)) {
+                $settings->h5pcustomcssurl = $customcss['cssurl']->out() . '?ver=' . $customcss['cssversion'];
+            }
+        }
+
         return $settings;
     }
 
@@ -571,6 +590,8 @@ class api {
                 'CoreUserDelegate_AddonBadges:account' => new lang_string('badges', 'badges'),
                 'CoreUserDelegate_AddonBlog:account' => new lang_string('blog', 'blog'),
                 '$mmSideMenuDelegate_mmaCompetency' => new lang_string('myplans', 'tool_lp'),
+                'CoreUserDelegate_CorePolicy' => new lang_string('policiesagreements', 'tool_policy'),
+                'CoreUserDelegate_CoreDataPrivacy' => new lang_string('pluginname', 'tool_dataprivacy'),
                 'NoDelegate_SwitchAccount' => new lang_string('switchaccount', 'tool_mobile'),
             ),
             "$course" => array(
@@ -768,7 +789,7 @@ class api {
      *
      * @return array Subscription information
      */
-    public static function get_subscription_information() : ?array {
+    public static function get_subscription_information(): ?array {
         global $CFG;
 
         // Use session cache to prevent multiple requests.
