@@ -418,7 +418,7 @@ class datalib_test extends \advanced_testcase {
         $this->assertSame('folder', $cm->modname);
         $this->assertSame($folder1a->id, $cm->instance);
         $this->assertSame($folder1a->course, $cm->course);
-        $this->assertObjectNotHasAttribute('sectionnum', $cm);
+        $this->assertObjectNotHasProperty('sectionnum', $cm);
 
         $this->assertEquals($cm, get_coursemodule_from_id('', $folder1a->cmid));
         $this->assertEquals($cm, get_coursemodule_from_id('folder', $folder1a->cmid, $course1->id));
@@ -483,7 +483,7 @@ class datalib_test extends \advanced_testcase {
         $this->assertSame('folder', $cm->modname);
         $this->assertSame($folder1a->id, $cm->instance);
         $this->assertSame($folder1a->course, $cm->course);
-        $this->assertObjectNotHasAttribute('sectionnum', $cm);
+        $this->assertObjectNotHasProperty('sectionnum', $cm);
 
         $this->assertEquals($cm, get_coursemodule_from_instance('folder', $folder1a->id, $course1->id));
         $this->assertEquals($cm, get_coursemodule_from_instance('folder', $folder1a->id, 0));
@@ -553,17 +553,17 @@ class datalib_test extends \advanced_testcase {
         $this->assertSame('folder', $cm->modname);
         $this->assertSame($folder1a->id, $cm->instance);
         $this->assertSame($folder1a->course, $cm->course);
-        $this->assertObjectNotHasAttribute('sectionnum', $cm);
-        $this->assertObjectNotHasAttribute('revision', $cm);
-        $this->assertObjectNotHasAttribute('display', $cm);
+        $this->assertObjectNotHasProperty('sectionnum', $cm);
+        $this->assertObjectNotHasProperty('revision', $cm);
+        $this->assertObjectNotHasProperty('display', $cm);
 
         $cm = $modules[$folder1b->cmid];
         $this->assertSame('folder', $cm->modname);
         $this->assertSame($folder1b->id, $cm->instance);
         $this->assertSame($folder1b->course, $cm->course);
-        $this->assertObjectNotHasAttribute('sectionnum', $cm);
-        $this->assertObjectNotHasAttribute('revision', $cm);
-        $this->assertObjectNotHasAttribute('display', $cm);
+        $this->assertObjectNotHasProperty('sectionnum', $cm);
+        $this->assertObjectNotHasProperty('revision', $cm);
+        $this->assertObjectNotHasProperty('display', $cm);
 
         $modules = get_coursemodules_in_course('folder', $course1->id, 'revision, display');
         $this->assertCount(2, $modules);
@@ -572,9 +572,9 @@ class datalib_test extends \advanced_testcase {
         $this->assertSame('folder', $cm->modname);
         $this->assertSame($folder1a->id, $cm->instance);
         $this->assertSame($folder1a->course, $cm->course);
-        $this->assertObjectNotHasAttribute('sectionnum', $cm);
-        $this->assertObjectHasAttribute('revision', $cm);
-        $this->assertObjectHasAttribute('display', $cm);
+        $this->assertObjectNotHasProperty('sectionnum', $cm);
+        $this->assertObjectHasProperty('revision', $cm);
+        $this->assertObjectHasProperty('display', $cm);
 
         $modules = get_coursemodules_in_course('label', $course1->id);
         $this->assertCount(0, $modules);
@@ -842,11 +842,11 @@ class datalib_test extends \advanced_testcase {
         $this->assertEquals('user_a@example.com', $results[$userids[0]]->email);
         $this->assertEquals(1, $results[$userids[0]]->confirmed);
         $this->assertEquals('a_first', $results[$userids[0]]->firstname);
-        $this->assertObjectHasAttribute('firstnamephonetic', $results[$userids[0]]);
+        $this->assertObjectHasProperty('firstnamephonetic', $results[$userids[0]]);
 
         // Should not have the custom field or department because no context specified.
-        $this->assertObjectNotHasAttribute('department', $results[$userids[0]]);
-        $this->assertObjectNotHasAttribute('profile_field_specialid', $results[$userids[0]]);
+        $this->assertObjectNotHasProperty('department', $results[$userids[0]]);
+        $this->assertObjectNotHasProperty('profile_field_specialid', $results[$userids[0]]);
 
         // Check sorting.
         $results = get_users_listing('username', 'DESC');
@@ -867,8 +867,8 @@ class datalib_test extends \advanced_testcase {
         // specify a context AND have permissions.
         $results = get_users_listing('lastaccess', 'asc', 0, 0, '', '', '', '', null,
                 \context_system::instance());
-        $this->assertObjectNotHasAttribute('department', $results[$userids[0]]);
-        $this->assertObjectNotHasAttribute('profile_field_specialid', $results[$userids[0]]);
+        $this->assertObjectNotHasProperty('department', $results[$userids[0]]);
+        $this->assertObjectNotHasProperty('profile_field_specialid', $results[$userids[0]]);
         $this->setAdminUser();
         $results = get_users_listing('lastaccess', 'asc', 0, 0, '', '', '', '', null,
                 \context_system::instance());
